# The LearningOnline Network with CAPA
# Interface routines for Aspell
#
# $Id: lonspeller.pm,v 1.9 2004/12/02 20:54:26 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
######################################################################
######################################################################

package Apache::lonspeller;

use Apache::Constants qw(:common);
use Text::Aspell;
use Apache::lonlocal;
use Apache::lontexconvert();
use HTML::LCParser;
use strict;

my $speller;
my $insidelink;

sub spellcheck_language {
    if ($ENV{'form.lang'}) { return $ENV{'form.lang'}; }
    if (&mt('spellcheck_lang') ne 'spellcheck_lang') {
	return &mt('spellcheck_lang');
    }
    return 'en_US';
}

sub set_language {
    my $lang=&spellcheck_language();
    $speller->set_option('lang',$lang);
}

sub textsection {
    my $input=shift;
    my $output='';
    &set_language();
    foreach my $word (split(/\b/,$input)) {
	if (($word=~/\W/) || ($word=~/^(lt|gt|nbsp|amp)$/i)
	    || ($speller->check($word))) {
	    $output.=$word;
	} else {
	    my $suggestions=join(' ',$speller->suggest($word));
	    $suggestions=~s/\'/\\\'/gs;
	    if (($suggestions) && (!$insidelink)) {
		$output.='<a href="javascript:spellwin=window.open('.
		    &Apache::lonhtmlcommon::javascript_nothing().
		    ',\'spellwin\',\'height=140,width=280,resizable=yes,scrollbars=yes,location=no,menubar=no,toolbar=no\');'.
                    'spellwin.'.&Apache::lonhtmlcommon::javascript_docopen().';spellwin.document.writeln(\'<html><body><h3>'.$word.
                    '</h3>'.$suggestions.'</body></html>\');spellwin.document.close();spellwin.focus()">';
	    }
	    $output.='<font color="red">'.$word.'</font>';
	    if (($suggestions) && (!$insidelink)) { $output.='</a>'; }
	}
    }
    return $output;
}


sub markeduptext {
    my $input=shift;
    my $output='';
    my $parser=HTML::LCParser->new(\$input);
    $insidelink=0;
    my $token;
    while ($token=$parser->get_token) {
	if ($token->[0] eq 'T') {
	    $output.=&textsection($token->[1]);
	} elsif ($token->[0] eq 'S') {
	    $output.=$token->[4];
	    foreach my $tag ('m','script') {
		if ($token->[1] eq $tag) {
		    $output.=$parser->get_text('/'.$tag);
		}
	    }
	    if ($token->[1] eq 'a') {
		$insidelink=1;
	    }
	} elsif ($token->[0] eq 'E') {
	    $output.=$token->[2];
	    if ($token->[1] eq 'a') {
		$insidelink=0;
	    }
	}
    }
    $insidelink=0;
    return $output;
}

sub initspeller {
    unless (defined($speller)) {
	$speller = Text::Aspell->new;
	$speller->set_option('lang','en_US');
    }
    $insidelink=0;
}

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;

    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'});

    &initspeller();
    $r->print('<html><head><title>'.
	      &mt('Spell Checker').
	      '</title></head><body bgcolor="#DDDDDD">'.
	      &Apache::lontexconvert::msgtexconverted(
				     &markeduptext($ENV{'form.text'})).
	      '</body></html>');
    return OK;
}

BEGIN {
    &initspeller();
}

1;

__END__
