# The LearningOnline Network with CAPA
# Construction Space Directory Lister
#
# $Id: lonpubdir.pm,v 1.76 2005/01/24 21:55:20 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###

package Apache::lonpubdir;

use strict;
use Apache::File;
use File::Copy;
use Apache::Constants qw(:common :http :methods);
use Apache::loncacc;
use Apache::loncommon();
use Apache::lonhtmlcommon();
use Apache::lonlocal;
use Apache::lonmsg;
use Apache::lonmenu;
use Apache::lonnet;

sub handler {

  my $r=shift;

  my $fn;



  $fn = getEffectiveUrl($r);

  # Validate access to the construction space and get username@domain.

  my $uname;
  my $udom;

  ($uname,$udom)=
    &Apache::loncacc::constructaccess(
             $fn,$r->dir_config('lonDefDomain')); 
  unless (($uname) && ($udom)) {
     $r->log_reason($uname.' at '.$udom.
         ' trying to list directory '.$ENV{'form.filename'}.
         ' ('.$fn.') - not authorized', 
         $r->filename); 
     return HTTP_NOT_ACCEPTABLE;
  }

  # Remove trailing / from directory name.

  $fn=~s/\/$//;

  unless ($fn) { 
     $r->log_reason($ENV{'user.name'}.' at '.$ENV{'user.domain'}.
         ' trying to list empty directory', $r->filename); 
     return HTTP_NOT_FOUND;
  } 

# ----------------------------------------------------------- Start page output

  my $thisdisfn=$fn;
  $thisdisfn=~s/^\/home\/$uname\/public_html//;	# subdirectory part of
                                                # construction space. 
  my $docroot=$r->dir_config('lonDocRoot');     # Apache  londocument root.

  my $resdir=$docroot.'/res/'.$udom.'/'.$uname.$thisdisfn; # Resource directory
  my $targetdir=$udom.'/'.$uname.$thisdisfn; # Publiction target directory.
  my $linkdir='/priv/'.$uname.$thisdisfn;      # Full URL name of constr space.

  my %bombs=&Apache::lonmsg::all_url_author_res_msg($uname,$udom);

  &startpage($r, $uname, $udom, $thisdisfn);   # Put out the start of page.
  if ($ENV{'environment.remote'} eq 'off') {
      &dircontrols($r,$uname,$udom,$thisdisfn);    # Put out actions for directory, 
                                               # browse/upload + new file page.
  } else {
      &pubbuttons($r,$uname,$thisdisfn);
  }
  &resourceactions($r,$uname,$udom,$thisdisfn); #Put out form used for printing/deletion etc.

  my $numdir = 0;
  my $numres = 0;
  
  # Start off the directory table.
  $r->print('<h3>Directory Contents:</h3>');
  $r->print('<table border="0" cellspacing="2" cellpadding="2"><tr>'.
            '<th bgcolor="#DDDDDD">'.&mt('Type').'</th>'.
            '<th bgcolor="#DDDDDD">'.&mt('Actions').'</th>'.
            '<th bgcolor="#DDDDDD">'.&mt('Name').'</th>'.
            '<th bgcolor="#DDDDDD">'.&mt('Title').'</th>'.
	    '<th bgcolor="#DDDDDD">'.&mt('Status').'</th>'.
            '<th bgcolor="#DDDDDD">'.&mt('Last Modified').
	    '</th></tr>');

  my $filename;
  my $dirptr=16384;		# Mask indicating a directory in stat.cmode.

  opendir(DIR,$fn);
  my @files=sort {uc($a) cmp uc($b)} (readdir(DIR));
  foreach my $filename (@files) {
     my ($cdev,$cino,$cmode,$cnlink,
         $cuid,$cgid,$crdev,$csize,
         $catime,$cmtime,$cctime,
         $cblksize,$cblocks)=stat($fn.'/'.$filename);

     my $extension='';
     if ($filename=~/\.(\w+)$/) { $extension=$1; }
     if ($cmode&$dirptr) {
	 putdirectory($r, $thisdisfn, $linkdir, $filename, $cmtime,$targetdir,\%bombs,\$numdir);
     } elsif (&Apache::loncommon::fileembstyle($extension) ne 'hdn') {
	 putresource($r, $udom, $uname, $filename, $thisdisfn, $resdir, 
		     $targetdir, $linkdir, $cmtime,\%bombs,\$numres);
     } else {
	# "hidden" extension and not a directory, so hide it away.
     }
  }
  closedir(DIR);

  $r->print('</table></body></html>');
  return OK;  
}
#
#  Gets the effective URL of the request and returns it:
#    $effn = getEffectiveUrl($r);
#       $r  - The Apache Request object.
sub getEffectiveUrl {
    my $r = shift;
    my $fn;
    
    if ($ENV{'form.filename'}) {	# If a form filename is defined.
	$fn=$ENV{'form.filename'};
	#
	#   Replace the ~username of the URL with /home/username/public_html
	#   so that we don't have to worry about ~ expansion internally.
	#
	$fn=~s/^http\:\/\/[^\/]+\///;
        $fn=~s/^\///;
        $fn=~s/\~(\w+)/\/home\/$1\/public_html/;
	
	#  Remove trailing / strings (?) 
	
	$fn=~s/\/[^\/]+$//;
    } else {
	#   If no form is defined, use request filename.
	$fn = $r->filename();
	my $lonDocRoot=$r->dir_config('lonDocRoot');
	if ( $fn =~ /$lonDocRoot/ ) {
	    #internal authentication, needs fixup.
	    $fn = $r->uri(); # non users do not get the full path request
                             # through SCRIPT_FILENAME
	    $fn=~s|^/~(\w+)|/home/$1/public_html|;
	}
    }
    $fn=~s/\/+/\//g;
    return $fn;
}
#
#   Output the header of the page.  This includes:
#   - The HTML header 
#   - The H1/H3  stuff which includes the directory.
#
#     startpage($r, $uame, $udom, $thisdisfn);
#      $r     - The apache request object.
#      $uname - User name.
#      $udom  - Domain name the user is logged in under.
#      $thisdisfn - Displayable version of the filename.

sub startpage {
    my ($r, $uname, $udom, $thisdisfn) = @_;
    my $currdir = '/priv/'.$uname.$thisdisfn;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;

    $r->print('<html><head><title>LON-CAPA Construction Space</title></head>');

    my $pagetitle;
    my $formaction='/priv/'.$uname.$thisdisfn.'/';
    $formaction=~s/\/+/\//g;
    $pagetitle .= &Apache::loncommon::help_open_menu('','','','',3,'Authoring').
        '<font face="Arial, Helvetica, sans-serif" size="+1"><b>Construction Space</b>:</font>&nbsp;'.
        '<form name="dirs" method="post" action="'.$formaction.
        '" target="_parent"><tt><b>'.
        &Apache::lonhtmlcommon::crumbs($uname.$thisdisfn.'/','_top','/priv','','+1',1)."</b></tt><br />".
        &Apache::lonhtmlcommon::select_recent('construct','recent',
                 'this.form.action=this.form.recent.value;this.form.submit()').
              '</form>';
    &Apache::lonhtmlcommon::store_recent('construct',$formaction,$formaction);
    if ($ENV{'environment.remote'} eq 'off') {
	$ENV{'request.noversionuri'}=$currdir.'/';
	$r->print(&Apache::loncommon::bodytag('Construction Space',undef,undef,undef,undef,undef,$pagetitle));
    } else {
	$r->print($pagetitle);
    }
    my $pubdirscript=(<<ENDPUBDIRSCRIPT);
<script>
top.document.title = '$thisdisfn/ - LON-CAPA Construction Space';
// Store directory location for menu bar to find

parent.lastknownpriv='/~$uname$thisdisfn/';

// Confirmation dialogues

    function currdiract(theform) {
        if (theform.dirtask.options[theform.dirtask.selectedIndex].value == 'publish') {
            document.publishdir.filename.value = theform.filename.value
            pubdir(document.publishdir)
        }
        if (theform.dirtask.options[theform.dirtask.selectedIndex].value == 'publishsub') {
            document.publishdir.filename.value = theform.filename.value
            pubrecdir(document.publishdir)
        }
        if (theform.dirtask.options[theform.dirtask.selectedIndex].value == 'editcat') {
            top.location=theform.filename.value+'default.meta'
        }
        if (theform.dirtask.options[theform.dirtask.selectedIndex].value == 'printdir' ) {
            document.printdir.postdata.value=theform.filename.value
            document.printdir.submit();
        }
    }
  
    function pubdir(theform) {
	if (confirm('Publish complete directory?')) {
            forcepub(theform)
	    theform.submit();
        }
    }
    function pubrecdir(theform) {
	if (confirm('Publish directory and all subdirectories?')) {
            forcepub(theform);
            theform.pubrec.value='1';
	    theform.submit();
        }
    }

    function forcepub(theform) {
        if (confirm('Force publication of unmodified files? -  Cancel=No; OK=yes.')) {
            theform.forcerepub.value="ON";
        } 
    }

    function checkUpload(theform) {
        if (theform.file == '') {
            alert("Please use 'Browse..' to choose a file first, before uploading")
            return 
        }
        theform.submit()  
    }

    function SetPubDir(theform,printForm) {
        if (theform.diraction.options[theform.diraction.selectedIndex].value == "open") {
            top.location = theform.openname.value
            return
        }
        if (theform.diraction.options[theform.diraction.selectedIndex].value == "publish") {
            pubdir(theform)
        }
        if (theform.diraction.options[theform.diraction.selectedIndex].value == "publishsub") {
            pubrecdir(theform)
        }
        if (theform.diraction.options[theform.diraction.selectedIndex].value == "editcat") {
            top.location=theform.filename.value+'default.meta'
        }
        if (theform.diraction.options[theform.diraction.selectedIndex].value == "printdir") {
            theform.action = '/adm/printout'
            theform.postdata.value = theform.filename.value
            theform.submit()
        }
        return
    }
    function SetResChoice(theform) {
      var activity = theform.reschoice.options[theform.reschoice.selectedIndex].value
      if ((activity == 'rename') || (activity == 'copy') || (activity == 'move')) {
          changename(theform,activity)
      }
      if (activity == 'publish') {
          var pubform = document.pubresource
          pubform.filename.value = theform.filename.value
          pubform.submit()
      }
      if (activity == 'delete') {
          var delform = document.delresource
          delform.filename.value = theform.filename.value
          delform.submit()
      }
      if (activity == 'obsolete') {
          var pubform = document.pubresource
          pubform.filename.value = theform.filename.value
          alert("You will be taken to the publication page.\\nCheck the 'Obsolete' checkbox at the bottom of the page, and click 'Finalize Publication'.")
          pubform.submit()
      }
      if (activity == 'print') {
          document.printresource.postdata.value = theform.filename.value
          document.printresource.submit()
      }
      if (activity == 'retrieve') {
          document.retrieveres.filename.value = theform.filename.value
          document.retrieveres.submit()
      }
      return
    }
    function changename(theform,activity) {
        var newname=prompt('New Name');
        if (newname == "" || !newname)  {
            return
        }
        document.moveresource.newfilename.value = newname
        document.moveresource.filename.value = theform.filename.value
        document.moveresource.action.value = activity
        document.moveresource.submit();
    }
</script>
ENDPUBDIRSCRIPT
    $r->print($pubdirscript);

    if ((($uname ne $ENV{'user.name'}) || ($udom ne $ENV{'user.domain'})) &&
	$ENV{'environment.remote'} ne 'off') {
	$r->print('<h3>'.&mt('Co-Author').': '.$uname.' at '.$udom.
		  '</h3>');
    }
}

sub dircontrols {
    my ($r,$uname,$udom,$thisdisfn) = @_;
    $r->print(<<END);
        <table cellspacing="4" cellpadding="4" width="100%">
         <tr>
          <td bgcolor="#DDDDDD" align="middle"><font face="Arial, Helvetica, sans-serif" size="-1"><b>Actions for current directory</b></font></td>
          <td bgcolor="#DDDDDD" align="middle"><font face="Arial, Helvetica, sans-serif" size="-1"><b>Upload a new document</b></font></td>
          <td bgcolor="#DDDDDD" align="middle"><font face="Arial, Helvetica, sans-serif" size="-1"><b>Create a new directory or LON-CAPA document</b></font></td>
        </tr>
        <tr>
         <td bgcolor="#ccddaa" valign="top" align="center">
          <form name="curractions" method="post" action="">
           <select name="dirtask" onChange="currdiract(this.form)">
            <option>Select action</option>
            <option value="publish">Publish directory</option>
            <option value="publishsub">Publish with subdirectories</option>
            <option value="editcat">Edit catalog information</option>
            <option value="printdir">Print contents of directory</option>
           </select>
           <input type="hidden" name="filename" value="/~$uname$thisdisfn/" />
          </form>
          <form name="publishdir" method="post" action="/adm/publish" target="_parent">
           <input type="hidden" name="pubrec" value="" />
           <input type="hidden" name="filename" value="" />
           <input type="hidden" name="forcerepub" value="NO" />
          </form>
          <form name="printdir" method="post" action="/adm/printout" target="_parent">
           <input type="hidden" name="postdata" value="" />
          </form>
         </td>
         <td bgcolor="#ccddaa" valign="top" align="center">
	    <form name="upublisher" enctype="multipart/form-data" method="post" action="/adm/upload" target="_parent">
	      <input type="hidden" name="filename" value="/~$uname$thisdisfn/" />
	      <input type="file" name="upfile" size="20" />
	      <input type="button" value="Upload file"  onclick="checkUpload(this.form)" />
	    </form>
	 </td>
	 <td bgcolor="#ccddaa" align="center">
	    <form name="fileaction" method="post" action="/adm/cfile" target="_parent">
	      <nobr>
		<input type="hidden" name="filename" value="/~$uname$thisdisfn/" />
		  <select name="action">
		    <option>Select Action</option>
		    <option value="newfile">New file:</option>
		    <option value="newhtmlfile">New HTML file:</option>
		    <option value="newproblemfile">New problem:</option>
                    <option value="newpagefile">New assembled page:</option>
                    <option value="newsequencefile">New assembled sequence:</option>
                    <option value="newrightsfile">New custom rights file:</option>
                    <option value="newstyfile">New style file:</option>
		    <option value="newdir">New subdirectory:</option>
		  </select>&nbsp;<input type="text" name="newfilename" value="Type Name Here" onfocus="if (this.value == 'Type Name Here') this.value=''" />&nbsp;<input type="button" value="Go" onclick="document.fileaction.submit()" />
		 </nobr>
		</form>
	  </td>
         </tr>
        </table>
END
}

sub pubbuttons {
    my ($r,$uname,$thisdisfn) = @_;
    $r->print('<form method="post" action="/adm/publish" target="_parent">'.
              '<table><tr><td><input type="hidden" name="filename" value="/~'.
               $uname.$thisdisfn.'/" />'.
              '<input type="button" onClick="pubdir(this.form);" value="'.
&mt('Publish Directory').'" />'.
              '<input type="hidden" name="pubrec" value="" />'.
              '<input type="hidden" name="forcerepub" value="NO" />'.
              '<input type="button" onClick="pubrecdir(this.form);" value="'.
&mt('Publish Directory and Sub Directories').'" /></td><td>'.
'<input type="button" onClick="window.location='."'/~".
               $uname.$thisdisfn."/default.meta'".'" value="'.
&mt('Edit Directory Catalog Information').'" /></td></tr></table></form>');
}

sub resourceactions {
    my ($r,$uname,$udom,$thisdisfn) = @_;
    $r->print(<<END);
       <form name="moveresource" action="/adm/cfile" target="_parent" method="post">
         <input type="hidden" name="filename" value="" />
         <input type="hidden" name="newfilename" value="" />
         <input type="hidden" name="action" value="" />
       </form>
       <form name="delresource" action="/adm/cfile" target="_parent" method="post">
         <input type="hidden" name="filename" value="" />
         <input type="hidden" name="action" value="delete" />
       </form>
       <form name="pubresource" action="/adm/publish" target="_parent" method="post">
         <input type="hidden" name="pubrec" value="" />
         <input type="hidden" name="filename" value="" />
         <input type="hidden" name="forcerepub" value="NO" />
       </form>
       <form name="printresource" action="/adm/printout" target="_parent" method="post">
           <input type="hidden" name="postdata" value="" />
       </form>
       <form name="retrieveres" action="/adm/retrieve" target="_parent" method="post">
           <input type="hidden" name="filename" value="" />
       </form>
END
}

#
#   Get the title string or "[untitled]" if the file has no title metadata:
#   Without the latter substitution, it's impossible to examine metadata for
#   untitled resources.  Resources may be legitimately untitled, to prevent
#   searches from locating them.
#
#   $str = getTitleString($fullname);
#       $fullname - Fully qualified filename to check.
#
sub getTitleString {
    my $fullname = shift;
    my $title    = &Apache::lonnet::metadata($fullname, 'title');

    unless ($title) {
	$title = "[".&mt('untitled')."]";
    }
    return $title;
}

sub getCopyRightString {
    my $fullname = shift;
    return &Apache::lonnet::metadata($fullname, 'copyright');
}

sub getSourceRightString {
    my $fullname = shift;
    return &Apache::lonnet::metadata($fullname, 'sourceavail');
}
#
#  Put out a directory table row:
#    putdirectory(r, base, here, dirname, modtime)
#      r       - Apache request object.
#      reqfile - File in request.
#      here    - Where we are in directory tree.
#      dirname - Name of directory special file.
#      modtime - Encoded modification time.
# 
sub putdirectory {
    my ($r, $reqfile, $here, $dirname, $modtime, $resdir, $bombs, $numdir) = @_;
    # construct the display filename: the directory name unless ..:
    
    my $disfilename = $dirname;
    if ($dirname eq '..') {
	$disfilename = '<i>'.&mt('Parent Directory').'</i>';
    }
    unless ( (($dirname eq '..') && ($reqfile eq '')) || ($dirname eq '.')) {
	my $kaputt=0;
	foreach (keys %{$bombs}) {
	    if ($_=~m:^\Q$resdir\E/\Q$disfilename\E/:) { $kaputt=1; last; }
	}
	%Apache::lonpublisher::metadatafields=();
	%Apache::lonpublisher::metadatakeys=();
	my $construct=$here;
	$construct=~s:^/priv/(\w+)$:/home/$1/public_html:;
        my $dirpath = $here;
        $dirpath=~s:^/priv/:/~:;
	&Apache::lonpublisher::metaeval(&Apache::lonnet::getfile(
       				 $construct.'/'.$dirname.'/default.meta'
								 ));
        my $actionitem = '';
        if ($dirname eq '..') {
            $actionitem = 'Go to ...';
        } else {
            $actionitem = 
                    '<form name="dirselect_'.$$numdir.
                    '" action="/adm/publish" target="_parent">'.
                    '<select name="diraction" onChange="SetPubDir(this.form,document)">'.
                      '<option selected="selected">'.&mt('Select action').'</option>'.
                      '<option value="open">'.&mt('Open').'</option>'.
                      '<option value="publish">'.&mt('Publish').'</option>'.
                      '<option value="publishsub">'.&mt('Publish with subdirectories').'</option>'.
                      '<option value="editcat">'.&mt('Edit catalog information').'</option>'.
                      '<option value="printdir">'.&mt('Print directory').
                    '</select>'.
                     '<input type="hidden" name="filename" value="'.$dirpath.'/'.$dirname.'/" />'.
                     '<input type="hidden" name="openname" value="'.$here.'/'.$dirname.'/" />'.
                     '<input type="hidden" name="pubrec" value="" />'.
                     '<input type="hidden" name="forcerepub" value="" />'.
                     '<input type="hidden" name="postdata" value="" />'.
                   '</form>';
            $$numdir ++;
        }
	$r->print('<tr bgcolor="#CCCCFF">'.
		  '<td><img src="'.
		  $Apache::lonnet::perlvar{'lonIconsURL'}.'/folder_closed.gif" /></td>'.
		  '<td>'.$actionitem.'</td>'.
		  '<td><font face="arial"><a href="'.$here.'/'.$dirname.'/" target="_parent">'.
		  $disfilename.'</a></font></td>'.
		        '<td colspan="2">'.($kaputt?&Apache::lonhtmlcommon::authorbombs($resdir.'/'.$disfilename.'/'):'').$Apache::lonpublisher::metadatafields{'title'}.' <i>'.
		  $Apache::lonpublisher::metadatafields{'subject'}.'</i> '.
		  $Apache::lonpublisher::metadatafields{'keywords'}.'</td>'.
		  '<td>'.&Apache::lonlocal::locallocaltime($modtime).'</td>'.
		  "</tr>\n");
    }
    return OK;
}
#
#   Put a table row for a file resource.
#
sub putresource {
    my ($r, $udom, $uname, $filename, $thisdisfn, 
	$resdir, $targetdir, $linkdir,
	$cmtime,$bombs,$numres) = @_;
    my $pubstatus = 'unpublished';
    my $status=&mt('Unpublished');
    my $bgcolor='#FFAA99';
    my $title='&nbsp;';
    my $publish_button=&mt('Publish');
#    my $action_buttons=
#        '<br /><a target="_parent" href="/adm/cfile?action=delete&filename=/~'.
#	$uname.'/'.$thisdisfn.'/'.$filename.'">'.
#	&mt('Delete').'</a>';
    if (-e $resdir.'/'.$filename) {
	my ($rdev,$rino,$rmode,$rnlink,
	    $ruid,$rgid,$rrdev,$rsize,
	    $ratime,$rmtime,$rctime,
	    $rblksize,$rblocks)=stat($resdir.'/'.$filename);
	$publish_button=&mt('Re-publish');
	if ($rmtime>=$cmtime) {
            $pubstatus = 'published';
	    $status=&mt('Published').'<br />'.
		&mt(&getCopyRightString($targetdir.'/'.$filename)).' '.
		&mt(&getSourceRightString($targetdir.'/'.$filename));
	    $bgcolor='#CCFF88';
	    if (&Apache::lonnet::metadata($targetdir.'/'.$filename,'obsolete')) {
                $pubstatus = 'obsolete';
		$status=&mt('Obsolete');
                $bgcolor='#AAAAAA';
            }
#	    } else {
#		$action_buttons='';
#	    }
	    $title='<a href="/res/'.$targetdir.'/'.$filename.
		'.meta" target=cat>'.
		&getTitleString($targetdir.'/'.$filename).'</a>';
	} else {
            $pubstatus = 'modified';
	    $status=&mt('Modified').'<br />'.
		&mt(&getCopyRightString($targetdir.'/'.$filename)).' '.
		&mt(&getSourceRightString($targetdir.'/'.$filename));
            $bgcolor='#FFFF77';
#	    $action_buttons='';
	    $title='<a href="/res/'.$targetdir.'/'.$filename.'.meta" target=cat>'.
		&getTitleString($targetdir.'/'.$filename).'</a>';
	    if (&Apache::loncommon::fileembstyle(($filename=~/\.(\w+)$/)) eq 'ssi') {
		$status.='<br /><a href="/adm/diff?filename=/~'.$uname.
		    $thisdisfn.'/'.$filename.
		    '&versiontwo=priv" target=cat>'.&mt('Diffs').'</a>';
	    }
	}
	$title.='<br /><a href="/~'.$uname.$thisdisfn.'/'.$filename.'.meta">'. 
	    ($$bombs{$targetdir.'/'.$filename}?'<img src="/adm/lonMisc/bomb.gif" border="0" />':'Edit Metadata').'</a>';
	$status.='<br><a href="/adm/retrieve?filename=/~'.$uname.
	    $thisdisfn.'/'.$filename.'" target="_parent">'.&mt('Retrieve').'</a>';
    }
    my $editlink='';
    my $editlink2='';
    if ($filename=~/\.(xml|html|htm|xhtml|xhtm|sty)$/) {
	$editlink=' <br />(<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_parent">'.&mt('Edit').'</a>)';
    }
    if ($filename=~/\.(problem|exam|quiz|assess|survey|form|library)$/) {
	$editlink=' (<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_parent">'.&mt('EditXML').'</a>)';
	$editlink2=' <br />(<a href="'.$linkdir.'/'.$filename.'?forceColoredit=1" target="_parent">'.&mt('Edit').'</a>)';
    }
    if ($filename=~/\.(zip|tar|bz2|gz|tar.gz|tar.bz2|tgz)$/) {
	$editlink=' (<a target="_parent" href="/adm/cfile?decompress=/~'.
	      $uname.$thisdisfn.'/'.$filename.'">'.&mt('Decompress').'</a>)';
    }
    my $pub_select = '';
    &create_pubselect($r,\$pub_select,$udom,$uname,$thisdisfn,$filename,$resdir,$pubstatus,$publish_button,$numres);
    $r->print('<tr bgcolor="'.$bgcolor.'">'.
	      '<td>'.($filename=~/[\#\~]$/?'&nbsp;':
		      '<img src="'.&Apache::loncommon::icon($filename).'" /></td>').
              '<td>'.$pub_select.'</td>'.
	      '<td><font face="arial">'.
	      '<a href="'.$linkdir.'/'.$filename.'" target="_parent">'.
               $filename.'</a></font>'.$editlink2.$editlink.
	      '</td>'.
	      '<td>'.$title.'</td>'.
	      '<td>'.$status.'</td>'.
	      '<td>'.&Apache::lonlocal::locallocaltime($cmtime).'</td>'.
	      "</tr>\n");
    return OK;
}

sub create_pubselect {
    my ($r,$pub_select,$udom,$uname,$thisdisfn,$filename,$resdir,$pubstatus,$publish_button,$numres) = @_;
    $$pub_select = '
<form name="resselect_'.$$numres.'" action="">
<select name="reschoice"  onChange="SetResChoice(this.form)">
<option>'.&mt('Select action').
'<option value="copy"/>'.&mt('Copy');
    if ($pubstatus eq 'obsolete' || $pubstatus eq 'unpublished') {
        $$pub_select .= 
'<option value="rename"/>'.&mt('Rename').
'<option value="move"/>'.&mt('Move').
'<option value="delete"/>'.&mt('Delete');
    } else {
        $$pub_select .= '
<option value="obsolete"/>'.&mt('Mark obsolete');
    }
# check for versions
    my $versions = &check_for_versions($r,'/'.$filename,$udom,$uname);
    if ($versions > 0) {
        $$pub_select .='
<option value="retrieve"/>'.&mt('Retrieve old version');
    }
    $$pub_select .= '
<option value="publish"/>'.$publish_button.
'<option value="print"/>'.&mt('Print').
'</select>
<input type="hidden" name="filename" value="/~'.
 $uname.$thisdisfn.'/'.$filename.'"></form>';
    $$numres ++;
}

sub check_for_versions {
    my ($r,$fn,$udom,$uname) = @_;
    my $versions = 0;
    my $docroot=$r->dir_config('lonDocRoot');
    my $resfn=$docroot.'/res/'.$udom.'/'.$uname.$fn;
    my $resdir=$resfn;
    $resdir=~s/\/[^\/]+$/\//;
    $fn=~/\/([^\/]+)\.(\w+)$/;
    my $main=$1;
    my $suffix=$2;
    opendir(DIR,$resdir);
    while (my $filename=readdir(DIR)) {
        if ($filename=~/^\Q$main\E\.(\d+)\.\Q$suffix\E$/) {
            $versions ++;        
        }
    }
    return $versions;
}

#
#   Categorize files in the directory.
#   For each file in a list of files in a file directory, 
#   the  file categorized as one of:
#    - directory  
#    - sequence
#    - problem 
#    - Other resource.
#
#   For each file the modification date is determined as well.
#   Returned is a list of sublists:
#    (directories, sequences, problems, other)
#   each of the sublists contains entries of the following form (sorted by
#   filename):
#     (filename, typecode, lastmodtime)
#
#   $list = CategorizeFiles($location, $files)
#       $location   - Directory in which the files live (relative to our
#                     execution.
#       $files      - list of files.
#
sub CategorizeFiles {
    my $location = shift;
    my $files    = shift;
}

1;
__END__

=head1 NAME

Apache::lonpubdir - Construction space directory lister

=head1 SYNOPSIS

Invoked (for various locations) by /etc/httpd/conf/srm.conf:

 <LocationMatch "^/\~.*/$">
 PerlAccessHandler       Apache::loncacc
 SetHandler perl-script
 PerlHandler Apache::lonpubdir
 ErrorDocument     403 /adm/login
 ErrorDocument     404 /adm/notfound.html
 ErrorDocument     406 /adm/unauthorized.html
 ErrorDocument	  500 /adm/errorhandler
 </LocationMatch>

 <Location /adm/pubdir>
 PerlAccessHandler       Apache::lonacc
 SetHandler perl-script
 PerlHandler Apache::lonpubdir
 ErrorDocument     403 /adm/login
 ErrorDocument     404 /adm/notfound.html
 ErrorDocument     406 /adm/unauthorized.html
 ErrorDocument	  500 /adm/errorhandler
 </Location>

=head1 INTRODUCTION

This module publishes a directory of files.

This is part of the LearningOnline Network with CAPA project
described at http://www.lon-capa.org.

=head1 HANDLER SUBROUTINE

This routine is called by Apache and mod_perl.

=over 4

=item *

read in information

=item *

start page output

=item *

run through list of files and attempt to publish unhidden files

=back

=cut
